
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v5.034.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "Catmandu modules for working with statistical data",
  "dist_author" => [
    "Patrick Hochstenbach, C<< <patrick.hochstenbach at ugent.be> >>"
  ],
  "dist_name" => "Catmandu-Stat",
  "dist_version" => "0.03",
  "license" => "perl",
  "module_name" => "Catmandu::Stat",
  "recommends" => {},
  "recursive_test_files" => 1,
  "requires" => {
    "Catmandu" => "0.9301",
    "Catmandu::Exporter::Table" => 0,
    "List::Util" => "1.42",
    "Statistics::Basic" => "1.6611",
    "perl" => "v5.10.1"
  },
  "script_files" => [],
  "test_requires" => {
    "Test::Deep" => "0.112",
    "Test::Exception" => "0.32",
    "Test::More" => "1.001003",
    "Test::Pod" => 0
  }
);


my %fallback_build_requires = (
  "Module::Build" => "0.28",
  "Test::Deep" => "0.112",
  "Test::Exception" => "0.32",
  "Test::More" => "1.001003",
  "Test::Pod" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);

$build->create_build_script;
